import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
  Config,
} from '@midwayjs/decorator';
import {
  BaseController,
  FrameworkErrorEnum,
  ServiceError,
} from 'free-midway-component';
import { UserService } from '../service/user.service';
import {
  CDTO_user,
  CVO_user,
  FVO_user,
  QDTO_user,
  QVO_user,
  UDTO_user,
  UVAO_user,
  LoginDTO,
  ResetPasswordDTO,
  LoginVO,
} from '../interface/user.interface';
import {
  ApiOperation,
  ApiQuery,
  ApiResponse,
  ApiTags,
} from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { UserUtilService } from '../service/util/user.util';
import { JwtService } from '@midwayjs/jwt';
@ApiTags(['用户'])
@Controller('/user')
export class UserController extends BaseController {
  @Inject()
  service: UserService;

  @Inject()
  jwtService: JwtService;

  @Inject()
  userUtil: UserUtilService;

  @Config('jwt.secret')
  jwtSecret;

  @Config('jwt.expiresIn')
  jwtExpiresIn;

  @Post('/login')
  @Validate()
  async login(@Body() body: LoginDTO) {
    const data = await this.service.findUserToLogin(body);
    const token = new LoginVO(data);
    return token;
  }

  @Post('/rp')
  @ApiOperation({ description: '修改密码' })
  async resetPassword(@Body() body: ResetPasswordDTO) {
    if (!body._id)
      throw new ServiceError('未找到用户信息', FrameworkErrorEnum.NOT_LOGIN);
    // 查询用户
    const user = await this.service.fetch(body._id);
    if (!user)
      throw new ServiceError('未找到用户', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 修改密码
    await this.service.updateOne(body._id, { password: body.password });
    // 返回结果
    return 'ok';
  }

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_user })
  async create(@Body() data: CDTO_user) {
    // 检查账号
    await this.userUtil.checkPhoneAndPid(data);
    const dbData = await this.service.create(data);
    const result = new CVO_user(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_user })
  async query(
    @Query() filter: QDTO_user,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    const list = await this.service.query(filter, { skip, limit });
    const data = [];
    for (const i of list) {
      const newData = new QVO_user(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_user })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_user(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_user })
  async update(@Param('id') id: string, @Body() body: UDTO_user) {
    // 检查账号
    await this.userUtil.checkUpdateCardAndPid(id, body);
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }

  @Get('/person')
  async person(filter: any) {
    const list = await this.service.person(filter);
    return list;
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
